@@ -4,7 +4,7 @@ import numpy as np
 
 from scipy.sparse.linalg import lsqr
 from pylops import LinearOperator
-from pylops.basicoperators import Diagonal
+from pylops.basicoperators import Diagonal, Identity
 from pylops.optimization.leastsquares import NormalEquationsInversion, \
     RegularizedInversion
 
@@ -69,14 +69,12 @@ def _softthreshold(x, thresh):
         Tresholded vector
 
     """
-    #if np.iscomplexobj(x):
-    #    # https://stats.stackexchange.com/questions/357339/soft-thresholding-
-    #    # for-the-lasso-with-complex-valued-data
-    #    x1 = np.maximum(np.abs(x) - thresh, 0.) * np.exp(1j * np.angle(x))
-    #else:
-    #    x1 = np.maximum(np.abs(x)-thresh, 0.) * np.sign(x)
-    x1 = x - thresh * x / np.abs(x)
-    x1[np.abs(x) <= thresh] = 0
+    if np.iscomplexobj(x):
+        # https://stats.stackexchange.com/questions/357339/soft-thresholding-
+        # for-the-lasso-with-complex-valued-data
+        x1 = np.maximum(np.abs(x) - thresh, 0.) * np.exp(1j * np.angle(x))
+    else:
+        x1 = np.maximum(np.abs(x)-thresh, 0.) * np.sign(x)
     return x1
 
 def _halfthreshold(x, thresh):
@@ -187,35 +185,124 @@ def _halfthreshold_percentile(x, perc):
     #return _halfthreshold(x, (2. / 3. * thresh) ** (1.5))
     return _halfthreshold(x, (4. / 54 ** (1. / 3.) * thresh) ** 1.5)
 
-def _shrinkage(x, thresh):
-    r"""Shrinkage.
-
-    Applies shrinkage to vector ``x``.
-
-    Parameters
-    ----------
-    x : :obj:`numpy.ndarray`
-        Vector
-    thresh : :obj:`float`
-        Threshold
+def _IRLS_data(Op, data, nouter, threshR=False, epsR=1e-10,
+               epsI=1e-10, x0=None, tolIRLS=1e-10,
+               returnhistory=False, **kwargs_solver):
+    r"""Iteratively reweighted least squares with L1 data term
     """
-    xabs = np.abs(x)
-    return x/(xabs+1e-10) * np.maximum(xabs - thresh, 0)
+    if x0 is not None:
+        data = data - Op * x0
+    if returnhistory:
+        xinv_hist = np.zeros((nouter + 1, Op.shape[1]))
+        rw_hist = np.zeros((nouter + 1, Op.shape[0]))
+
+    # first iteration (unweighted least-squares)
+    xinv = NormalEquationsInversion(Op, None, data, epsI=epsI,
+                                    returninfo=False,
+                                    **kwargs_solver)
+    r = data - Op * xinv
+    if returnhistory:
+        xinv_hist[0] = xinv
+    for iiter in range(nouter):
+        # other iterations (weighted least-squares)
+        xinvold = xinv.copy()
+        if threshR:
+            rw = 1. / np.maximum(np.abs(r), epsR)
+        else:
+            rw = 1. / (np.abs(r) + epsR)
+        rw = rw / rw.max()
+        R = Diagonal(rw)
+        xinv = NormalEquationsInversion(Op, [], data, Weight=R,
+                                        epsI=epsI,
+                                        returninfo=False,
+                                        **kwargs_solver)
+        r = data - Op * xinv
+        # save history
+        if returnhistory:
+            rw_hist[iiter] = rw
+            xinv_hist[iiter + 1] = xinv
+        # check tolerance
+        if np.linalg.norm(xinv - xinvold) < tolIRLS:
+            nouter = iiter
+            break
+
+    # adding initial guess
+    if x0 is not None:
+        xinv = x0 + xinv
+        if returnhistory:
+            xinv_hist = x0 + xinv_hist
+
+    if returnhistory:
+        return xinv, nouter, xinv_hist[:nouter + 1], rw_hist[:nouter + 1]
+    else:
+        return xinv, nouter
+
+
+def _IRLS_model(Op, data, nouter, threshR=False, epsR=1e-10,
+                epsI=1e-10, x0=None, tolIRLS=1e-10,
+                returnhistory=False, **kwargs_solver):
+    r"""Iteratively reweighted least squares with L1 model term
+    """
+    if x0 is not None:
+        data = data - Op * x0
+    if returnhistory:
+        xinv_hist = np.zeros((nouter + 1, Op.shape[1]))
+        rw_hist = np.zeros((nouter + 1, Op.shape[0]))
+
+    Iop = Identity(data.size, dtype=data.dtype)
+    # first iteration (unweighted least-squares)
+    xinv = Op.H @ lsqr(Op @ Op.H + (epsI**2) * Iop, data, **kwargs_solver)[0]
+    if returnhistory:
+        xinv_hist[0] = xinv
+    for iiter in range(nouter):
+        # other iterations (weighted least-squares)
+        xinvold = xinv.copy()
+        rw = np.abs(xinv)
+        rw = rw / rw.max()
+        R = Diagonal(rw)
+        xinv = R @ Op.H @ lsqr(Op @ R @ Op.H + epsI**2 * Iop,
+                               data, **kwargs_solver)[0]
+        # save history
+        if returnhistory:
+            rw_hist[iiter] = rw
+            xinv_hist[iiter + 1] = xinv
+        # check tolerance
+        if np.linalg.norm(xinv - xinvold) < tolIRLS:
+            nouter = iiter
+            break
+
+    # adding initial guess
+    if x0 is not None:
+        xinv = x0 + xinv
+        if returnhistory:
+            xinv_hist = x0 + xinv_hist
+
+    if returnhistory:
+        return xinv, nouter, xinv_hist[:nouter + 1], rw_hist[:nouter + 1]
+    else:
+        return xinv, nouter
+
 
 
 def IRLS(Op, data, nouter, threshR=False, epsR=1e-10,
          epsI=1e-10, x0=None, tolIRLS=1e-10,
-         returnhistory=False, **kwargs_cg):
+         returnhistory=False, kind='data',
+         **kwargs_solver):
     r"""Iteratively reweighted least squares.
 
-    Solve an optimization problem with :math:`L1` cost function given the
-    operator ``Op`` and data ``y``. The cost function is minimized by
-    iteratively solving a weighted least squares problem with the weight at
-    iteration :math:`i` being based on the data residual at iteration
-    :math:`i+1`.
+    Solve an optimization problem with :math:`L1` cost function (data IRLS)
+    or :math:`L1` regularization term (model IRLS) given the operator ``Op``
+    and data ``y``.
 
-    The IRLS solver is robust to *outliers* since the L1 norm given less
-    weight to large residuals than L2 norm does.
+    In the *data IRLS*, the cost function is minimized by iteratively solving a
+    weighted least squares problem with the weight at iteration :math:`i`
+    being based on the data residual at iteration :math:`i-1`. This IRLS solver
+    is robust to *outliers* since the L1 norm given less weight to large
+    residuals than L2 norm does.
+
+    Similarly in the *model IRLS*, the weight at at iteration :math:`i`
+    is based on the model at iteration :math:`i-1`. This IRLS solver inverts
+    for a sparse model vector.
 
     Parameters
     ----------
@@ -239,9 +326,12 @@ def IRLS(Op, data, nouter, threshR=False, epsR=1e-10,
         at subsequent iterations is smaller than ``tolIRLS``
     returnhistory : :obj:`bool`, optional
         Return history of inverted model for each outer iteration of IRLS
-    **kwargs_cg
+    kind : :obj:`str`, optional
+        Kind of solver (``data`` or ``model``)
+    **kwargs_solver
         Arbitrary keyword arguments for
-        :py:func:`scipy.sparse.linalg.cg` solver
+        :py:func:`scipy.sparse.linalg.cg` solver for data IRLS and
+        :py:func:`scipy.sparse.linalg.lsqr` solver for model IRLS
 
     Returns
     -------
@@ -256,19 +346,19 @@ def IRLS(Op, data, nouter, threshR=False, epsR=1e-10,
 
     Notes
     -----
-    Solves the following optimization problem for the operator
+    *Data IRLS* solves the following optimization problem for the operator
     :math:`\mathbf{Op}` and the data :math:`\mathbf{d}`:
 
     .. math::
         J = ||\mathbf{d} - \mathbf{Op} \mathbf{x}||_1
 
-    by a set of outer iterations which require to repeateadly solve a
+    by a set of outer iterations which require to repeatedly solve a
     weighted least squares problem of the form:
 
     .. math::
         \mathbf{x}^{(i+1)} = \operatorname*{arg\,min}_\mathbf{x} ||\mathbf{d} -
-        \mathbf{Op} \mathbf{x}||_{2, \mathbf{R}^{(i)}} +
-        \epsilon_I^2 ||\mathbf{x}||
+        \mathbf{Op} \mathbf{x}||_{2, \mathbf{R}^{(i)}}^2 +
+        \epsilon_I^2 ||\mathbf{x}||_2^2
 
     where :math:`\mathbf{R}^{(i)}` is a diagonal weight matrix
     whose diagonal elements at iteration :math:`i` are equal to the absolute
@@ -289,55 +379,42 @@ def IRLS(Op, data, nouter, threshR=False, epsR=1e-10,
     :math:`\epsilon_R` is the user-defined stabilization/thresholding
     factor [1]_.
 
+    Similarly *model IRLS* solves the following optimization problem for the
+    operator :math:`\mathbf{Op}` and the data :math:`\mathbf{d}`:
+
+    .. math::
+        J = ||\mathbf{x}||_1 \quad s.t. \quad
+        \mathbf{d} = \mathbf{Op} \mathbf{x}
+
+    by a set of outer iterations which require to repeatedly solve a
+    weighted least squares problem of the form:
+
+    .. math::
+        \mathbf{x}^{(i+1)} = \operatorname*{arg\,min}_\mathbf{x}
+        ||\mathbf{x}||_{2, \mathbf{R}^{(i)}}^2 \quad s.t. \quad
+        \mathbf{d} = \mathbf{Op} \mathbf{x}
+
+    where :math:`\mathbf{R}^{(i)}` is a diagonal weight matrix
+    whose diagonal elements at iteration :math:`i` are equal to the absolute
+    inverses of the model vector :math:`\mathbf{x}^{(i)}` at iteration
+    :math:`i`. More specifically the j-th element of the diagonal of
+    :math:`\mathbf{R}^{(i)}` is
+
+    .. math::
+        R^{(i)}_{j,j} = |x^{(i)}_j|
+
     .. [1] https://en.wikipedia.org/wiki/Iteratively_reweighted_least_squares
 
     """
-    if x0 is not None:
-        data = data - Op * x0
-    if returnhistory:
-        xinv_hist = np.zeros((nouter+1, Op.shape[1]))
-        rw_hist = np.zeros((nouter+1, Op.shape[0]))
-
-    # first iteration (unweighted least-squares)
-    xinv = NormalEquationsInversion(Op, None, data, epsI=epsI,
-                                    returninfo=False,
-                                    **kwargs_cg)
-    r = data-Op*xinv
-    if returnhistory:
-        xinv_hist[0] = xinv
-    for iiter in range(nouter):
-        # other iterations (weighted least-squares)
-        xinvold = xinv.copy()
-        if threshR:
-            rw = 1./np.maximum(np.abs(r), epsR)
-        else:
-            rw = 1./(np.abs(r)+epsR)
-        rw = rw / rw.max()
-        R = Diagonal(rw)
-        xinv = NormalEquationsInversion(Op, [], data, Weight=R,
-                                        epsI=epsI,
-                                        returninfo=False,
-                                        **kwargs_cg)
-        r = data-Op*xinv
-        # save history
-        if returnhistory:
-            rw_hist[iiter] = rw
-            xinv_hist[iiter+1] = xinv
-        # check tolerance
-        if np.linalg.norm(xinv - xinvold) < tolIRLS:
-            nouter = iiter
-            break
-
-    # adding initial guess
-    if x0 is not None:
-        xinv = x0 + xinv
-        if returnhistory:
-            xinv_hist = x0 + xinv_hist
-
-    if returnhistory:
-        return xinv, nouter, xinv_hist[:nouter+1], rw_hist[:nouter+1]
+    if kind == 'data':
+        solver = _IRLS_data
+    elif kind == 'model':
+        solver = _IRLS_model
     else:
-        return xinv, nouter
+        raise NotImplementedError('kind must be data or model')
+    return solver(Op, data, nouter, threshR=threshR, epsR=epsR,
+                  epsI=epsI, x0=x0, tolIRLS=tolIRLS,
+                  returnhistory=returnhistory, **kwargs_solver)
 
 
 def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
@@ -355,13 +432,14 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
         Operator to invert
     data : :obj:`numpy.ndarray`
         Data
-    niter_outer : :obj:`int`
+    niter_outer : :obj:`int`, optional
         Number of iterations of outer loop
-    niter_inner : :obj:`int`
-        Number of iterations of inner loop
+    niter_inner : :obj:`int`, optional
+        Number of iterations of inner loop. By choosing ``niter_inner=0``, the
+        Matching Pursuit (MP) algorithm is implemented.
     sigma : :obj:`list`
         Maximum L2 norm of residual. When smaller stop iterations.
-    normalizecols : :obj:`list`
+    normalizecols : :obj:`list`, optional
         Normalize columns (``True``) or not (``False``). Note that this can be
         expensive as it requires applying the forward operator
         :math:`n_{cols}` times to unit vectors (i.e., containing 1 at
@@ -393,7 +471,7 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
 
     .. math::
             ||\mathbf{x}||_0 \quad  subj. to \quad
-            ||\mathbf{Op}\mathbf{x}-\mathbf{b}||_2 <= \sigma,
+            ||\mathbf{Op}\mathbf{x}-\mathbf{b}||_2^2 <= \sigma,
 
     using Orthogonal Matching Pursuit (OMP). This is a very
     simple iterative algorithm which applies the following step:
@@ -402,13 +480,26 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
         \Lambda_k = \Lambda_{k-1} \cup \{ arg max_j
         |\mathbf{Op}_j^H \mathbf{r}_k| \} \\
         \mathbf{x}_k =  \{ arg min_{\mathbf{x}}
-        ||\mathbf{Op}_{\Lambda_k} \mathbf{x} - \mathbf{b}||_2
+        ||\mathbf{Op}_{\Lambda_k} \mathbf{x} - \mathbf{b}||_2^2
+
+    Note that by choosing ``niter_inner=0`` the basic Matching Pursuit (MP)
+    algorithm is implemented instead. In other words, instead of solving an
+    optimization at each iteration to find the best :math:`\mathbf{x}` for the
+    currently selected basis functions, the vector :math:`\mathbf{x}` is just
+    updated at the new basis function by taking directly the value from
+    the inner product :math:`\mathbf{Op}_j^H \mathbf{r}_k`.
+
+    In this case it is highly reccomended to provide a normalized basis
+    function. If different basis have different norms, the solver is likely
+    to diverge. Similar observations apply to OMP, even though mild unbalancing
+    between the basis is generally properly handled.
 
     """
     Op = LinearOperator(Op)
     if show:
         tstart = time.time()
-        print('OMP optimization\n'
+        algname = 'OMP optimization\n' if niter_inner > 0 else 'MP optimization\n'
+        print(algname +
               '-----------------------------------------------------------------\n'
               'The Operator Op has %d rows and %d cols\n'
               'sigma = %.2e\tniter_outer = %d\tniter_inner = %d\n'
@@ -420,7 +511,7 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
         ncols = Op.shape[1]
         norms = np.zeros(ncols)
         for icol in range(ncols):
-            unit = np.zeros(ncols)
+            unit = np.zeros(ncols, dtype=Op.dtype)
             unit[icol] = 1
             norms[icol] = np.linalg.norm(Op.matvec(unit))
     if show:
@@ -428,31 +519,49 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
         head1 = '    Itn           r2norm'
         print(head1)
 
+    if niter_inner == 0:
+        x = []
     cols = []
     res = data.copy()
     cost = np.zeros(niter_outer + 1)
     cost[0] = np.linalg.norm(data)
     iiter = 0
     while iiter < niter_outer and cost[iiter] > sigma:
-        cres = np.abs(Op.rmatvec(res))
+        # compute inner products
+        cres = Op.rmatvec(res)
+        cres_abs = np.abs(cres)
         if normalizecols:
-            cres = cres / norms
-        # exclude columns already chosen by putting them negative
-        if iiter > 0:
-            cres[cols] = -1
+            cres_abs = cres_abs / norms
         # choose column with max cres
-        imax = np.argwhere(cres == np.max(cres)).ravel()
+        cres_max = np.max(cres_abs)
+        imax = np.argwhere(cres_abs == cres_max).ravel()
         nimax = len(imax)
         if nimax > 0:
             imax = imax[np.random.permutation(nimax)[0]]
         else:
             imax = imax[0]
-        cols.append(imax)
+        # update active set
+        if imax not in cols:
+            addnew = True
+            cols.append(imax)
+        else:
+            addnew = False
+            imax_in_cols = cols.index(imax)
 
         # estimate model for current set of columns
-        Opcol = Op.apply_columns(cols)
-        x = lsqr(Opcol, data, iter_lim=niter_inner)[0]
-        res = data - Opcol.matvec(x)
+        if niter_inner == 0:
+            # MP update
+            Opcol = Op.apply_columns([imax, ])
+            res -= Opcol.matvec([cres[imax], ])
+            if addnew:
+                x.append(cres[imax])
+            else:
+                x[imax_in_cols] += cres[imax]
+        else:
+            # OMP update
+            Opcol = Op.apply_columns(cols)
+            x = lsqr(Opcol, data, iter_lim=niter_inner)[0]
+            res = data - Opcol.matvec(x)
         iiter += 1
         cost[iiter] = np.linalg.norm(res)
         if show:
@@ -460,7 +569,7 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
                 msg = '%6g        %12.5e' % (iiter + 1, cost[iiter])
                 print(msg)
     xinv = np.zeros(Op.shape[1], dtype=Op.dtype)
-    xinv[cols] = x
+    xinv[cols] = np.array(x)
     if show:
         print('\nIterations = %d        Total time (s) = %.2f'
               % (iiter, time.time() - tstart))
@@ -1011,13 +1120,13 @@ def SPGL1(Op, data, SOp=None, tau=0, sigma=0, x0=None, **kwargs_spgl1):
 
         .. math::
             ||\mathbf{x}||_1 \quad  subj. to \quad
-            ||\mathbf{Op}\mathbf{S}^H\mathbf{x}-\mathbf{b}||_2 <= \sigma,
+            ||\mathbf{Op}\mathbf{S}^H\mathbf{x}-\mathbf{b}||_2^2 <= \sigma,
 
     while the second problem is the *l1-regularized least-squares or LASSO*
     problem and its cost function is
 
         .. math::
-            ||\mathbf{Op}\mathbf{S}^H\mathbf{x}-\mathbf{b}||_2 \quad  subj.
+            ||\mathbf{Op}\mathbf{S}^H\mathbf{x}-\mathbf{b}||_2^2 \quad  subj.
             to \quad  ||\mathbf{x}||_1  <= \tau
 
     .. [1] van den Berg E., Friedlander M.P., "Probing the Pareto frontier
@@ -1056,7 +1165,10 @@ def SplitBregman(Op, RegsL1, data, niter_outer=3, niter_inner=5, RegsL2=None,
     niter_outer : :obj:`int`
         Number of iterations of outer loop
     niter_inner : :obj:`int`
-        Number of iterations of inner loop
+        Number of iterations of inner loop of first step of the Split Bregman
+        algorithm. A small number of iterations is generally sufficient and
+        for many applications optimal efficiency is obtained when only one
+        iteration is performed.
     RegsL2 : :obj:`list`
         Additional L2 regularization operators
         (if ``None``, L2 regularization is not added to the problem)
@@ -1086,7 +1198,8 @@ def SplitBregman(Op, RegsL1, data, niter_outer=3, niter_inner=5, RegsL2=None,
         Display iterations log
     **kwargs_lsqr
         Arbitrary keyword arguments for
-        :py:func:`scipy.sparse.linalg.lsqr` solver
+        :py:func:`scipy.sparse.linalg.lsqr` solver used to solve the first
+        subproblem in the first step of the Split Bregman algorithm.
 
     Returns
     -------
@@ -1103,26 +1216,46 @@ def SplitBregman(Op, RegsL1, data, niter_outer=3, niter_inner=5, RegsL2=None,
     :math:`\mathbf{R_{L1,i}}`, respectively:
 
     .. math::
-        J = \mu/2 ||\textbf{d} - \textbf{Op} \textbf{x} |||_2 +
-        \sum_i \epsilon_{{R}_{L2,i}} ||\mathbf{d_{{R}_{L2,i}}} -
-        \mathbf{R_{L2,i}} \textbf{x} |||_2 +
+        J = \mu/2 ||\textbf{d} - \textbf{Op} \textbf{x} |||_2^2 +
+        \sum_i \epsilon_{{R}_{L2,i}}/2 ||\mathbf{d_{{R}_{L2,i}}} -
+        \mathbf{R_{L2,i}} \textbf{x} |||_2^2 +
         \sum_i || \mathbf{R_{L1,i}} \textbf{x} |||_1
 
     where :math:`\mu` and :math:`\epsilon_{{R}_{L2,i}}` are the damping factors
-    used to weight the different terms of the cost function.
+    used to weight the different L2 regularization terms of the cost function.
 
     The generalized Split Bergman algorithm is used to solve such cost
     function: the algorithm is composed of a sequence of unconstrained
-    inverse problems and Bregman updates. Note that the L1 terms are not
-    weighted in the  original cost function but are first converted into
-    constraints and then re-inserted in the cost function with Lagrange
-    multipliers :math:`\epsilon_{{R}_{L1,i}}`, which effectively act as
-    damping factors for those terms. See [1]_ for detailed derivation.
+    inverse problems and Bregman updates.
+
+    The original system of equations is initially converted into a constrained
+    problem:
+
+    .. math::
+        J = \mu/2 ||\textbf{d} - \textbf{Op} \textbf{x} |||_2^2 +
+        \sum_i \epsilon_{{R}_{L2,i}}/2 ||\mathbf{d_{{R}_{L2,i}}} -
+        \mathbf{R_{L2,i}} \textbf{x}||_2^2 +
+        \sum_i || \textbf{y}_i ||_1 \quad s.t \quad
+        \textbf{y}_i = \mathbf{R_{L1,i}} \textbf{x} \quad \forall i
+
+    and solved as follows:
+
+    .. math::
+        (\textbf{x}^{k+1}, \textbf{y}_i^{k+1}) =
+        \operatorname*{arg\,min}_{\mathbf{x}, \mathbf{y}_i}
+        ||\textbf{d} - \textbf{Op} \textbf{x} |||_2^2 +
+        \sum_i \epsilon_{{R}_{L2,i}}/2 ||\mathbf{d_{{R}_{L2,i}}} -
+        \mathbf{R_{L2,i}} \textbf{x}||_2^2 + \sum_i || \textbf{y}_i ||_1
+        \sum_i \epsilon_{{R}_{L1,i}}/2 ||\textbf{y}_i -
+        \mathbf{R_{L1,i}} \textbf{x} - \textbf{b}_i^k||_2^2
+
+    .. math::
+        \textbf{b}_i^{k+1}=\textbf{b}_i^k +
+        (\mathbf{R_{L1,i}} \textbf{x}^{k+1} - \textbf{y}^{k+1})
 
     The :py:func:`scipy.sparse.linalg.lsqr` solver and a fast shrinkage
-    algorithm are used within the inner loop to solve the unconstrained
-    inverse problem, and the same procedure is repeated ``niter_outer`` times
-    until convergence.
+    algorithm are used within a inner loop to solve the first step. The entire
+    procedure is repeated ``niter_outer`` times until convergence.
 
     .. [1] Goldstein T. and Osher S., "The Split Bregman Method for
        L1-Regularized Problems", SIAM J. on Scientific Computing, vol. 2(2),
@@ -1153,7 +1286,7 @@ def SplitBregman(Op, RegsL1, data, niter_outer=3, niter_inner=5, RegsL2=None,
     if nregsL2 > 0:
         Regs = RegsL2 + RegsL1
         if dataregsL2 is None:
-            dataregsL2 = [np.zeros(Op.shape[1])] * nregsL2
+            dataregsL2 = [np.zeros(Reg.shape[0]) for Reg in RegsL2]
     else:
         Regs = RegsL1
         dataregsL2 = []
@@ -1179,7 +1312,7 @@ def SplitBregman(Op, RegsL1, data, niter_outer=3, niter_inner=5, RegsL2=None,
                                         x0=x0 if restart else xinv,
                                         **kwargs_lsqr)
             # Shrinkage
-            d = [_shrinkage(RegsL1[ireg] * xinv + b[ireg], epsRL1s[ireg])
+            d = [_softthreshold(RegsL1[ireg] * xinv + b[ireg], epsRL1s[ireg])
                  for ireg in range(nregsL1)]
         # Bregman update
         b = [b[ireg] + tau * (RegsL1[ireg] * xinv - d[ireg]) for ireg in
